<?php

namespace NoccyLabs\MessageCatalog;

use PDO;
use Traversable;

class Message
{
    public static array $filters = [];

    private array $content;

    public function __construct(
        ?string ...$content
    )
    {
        $this->content = $content; // array_filter($content, fn($v) => !is_null($v));
    }

    public function getAllContent(): array
    {
        return $this->content;
    }

    public function getContent(string $type): ?string
    {
        if (isset($this->content[$type])) {
            return $this->content[$type];
        }
        throw new \RuntimeException();
    }

    public function getTransformed(array $context, array $filters = []): array
    {
        $result = [];
        foreach ($this->content as $type=>$content) {
            if ($content !== null)
                $result[$type] = $this->transform($content, $context, [ ...self::$filters, ...$filters ]);
        }
        return $result;
    }

    public function transform(string $content, array $context, array $filters): string
    {
        $content = preg_replace_callback('<%\{(.+?)\}>', function ($m) use ($context, $filters) {
            $key = $m[1];
            if (str_contains($key, '|')) {
                $filt= explode('|', $key);
                $key = array_shift($filt);
            } else {
                $filt = null;
            }
            if (!isset($context[$key])) {
                throw new \RuntimeException("Template uses undefined slot {$key}");
            }
            if ($filt === null) {
                return $context[$key];
            }
            $value = $context[$key];
            foreach ($filt as $filtDef) {
                if (str_contains($filtDef,":")) {
                    [ $filtDef, $filtArgs ] = explode(":",$filtDef,2);
                    $filtArgs = str_getcsv($filtArgs);
                } else {
                    $filtArgs = [];
                }
                $filter = $filters[$filtDef]??null;
                if (!$filter)
                    throw new \RuntimeException("Undefined filter {$filtDef}");
                $value = $filter($value, ...$filtArgs);
            }
            return $value;
        }, $content);
        return $content;
    }
}
