<?php

namespace NoccyLabs\MessageCatalog;

use PHPUnit\Framework\Attributes\CoversClass;

#[CoversClass(Message::class)]
class MessageTest extends \PHPUnit\Framework\TestCase
{

    public function testCreatingMessageAndGettingAll()
    {
        $message = new Message(a:'aa', b:'bb', c:'cc');
        $this->assertEquals(['a'=>'aa', 'b'=>'bb', 'c'=>'cc'], $message->getAllContent());
    }

    public function testCreatingMessageAndGettingOne()
    {
        $message = new Message(a:'aa', b:'bb', c:'cc');
        $this->assertEquals('aa', $message->getContent('a'));
        $this->assertEquals('bb', $message->getContent('b'));
        $this->assertEquals('cc', $message->getContent('c'));
    }

    public function testTransformingMessages()
    {
        $message = new Message(text:"Hello %{who}");
        $transformed = $message->getTransformed([ 'who' => 'World' ]);
        $this->assertEquals("Hello World", $transformed['text']);
    }

    public function testTransformingWithFilters()
    {
        Message::$filters = [ 'lower' => fn($v) => strtolower($v) ];
        $filters = [ 'upper' => fn($v) => strtoupper($v) ];

        $message = new Message(text:"Hello %{who|upper}");
        $transformed = $message->getTransformed([ 'who' => 'World' ], $filters);
        $this->assertEquals("Hello WORLD", $transformed['text']);

        $message = new Message(text:"Hello %{who|lower}");
        $transformed = $message->getTransformed([ 'who' => 'World' ], $filters);
        $this->assertEquals("Hello world", $transformed['text']);
    }

    public function testTransformingWithFiltersAndArguments()
    {
        $filters = [ 'date' => fn($v,$f='Y-m-d H:i') => $v->format($f) ];

        $message = new Message(text:"%{date|date}");
        $transformed = $message->getTransformed([ 'date' => new \DateTime("2024-02-03 04:05:06") ], $filters);
        $this->assertEquals("2024-02-03 04:05", $transformed['text']);

        $message = new Message(text:"%{date|date:\"H:i:s\"}");
        $transformed = $message->getTransformed([ 'date' => new \DateTime("2024-02-03 04:05:06") ], $filters);
        $this->assertEquals("04:05:06", $transformed['text']);
    }

}
